﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;

namespace CodeReview.Binary
{
    /// <summary>
    /// Tags a check class as an equivalent for an existing check in an other library.
    /// </summary>
    /// <remarks>
    ///  This attribute is used to document the checks part of the code review that are
    ///  based on other libraries definitions.
    /// </remarks>
    [AttributeUsage(AttributeTargets.Class, AllowMultiple = true, Inherited = false)]
    public class CheckEquivalenceAttribute : Attribute
    {
        /// <summary>
        /// The code of the library.
        /// </summary>
        private readonly string library;

        /// <summary>
        /// The identifier of the check on the associated library.
        /// </summary>
        private readonly string checkId;

        /// <summary>
        /// Initializes a new instance of the CheckEquivalenceAttribute class.
        /// </summary>
        /// <param name="library">
        /// The code of the library.
        /// </param>
        /// <param name="checkId">
        /// The identifier of the check on the associated library.
        /// </param>
        public CheckEquivalenceAttribute(string library, string checkId)
        {
            this.library = library;
            this.checkId = checkId;
        }

        /// <summary>
        /// Gets the code of the library.
        /// </summary>
        public string Library
        {
            get { return this.library; }
        }

        /// <summary>
        /// Gets the identifier of the check on the associated library.
        /// </summary>
        public string CheckId
        {
            get { return this.checkId; }
        }
    }
}
