﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that out parameters are only used when overriding a method
    /// that requires such a parameter.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1045")]
    public class CR0023ParameterShouldNotUseRef : AbstractCheck
    {
        /// <summary>
        /// Initializes a new instance of the
        /// CR0023ParameterShouldNotUseRef class.
        /// </summary>
        public CR0023ParameterShouldNotUseRef()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on parameter parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParsedParameter += this.OnParsedParameter;
        }

        /// <summary>
        /// Called when a parameter is parsed.
        /// </summary>
        /// <param name="parameter">
        /// The parsed parameter.
        /// </param>
        private void OnParsedParameter(ParameterInfo parameter)
        {
            // Avoid return parameter
            if (string.IsNullOrEmpty(parameter.Name))
            {
                return;
            }

            if (!parameter.IsOut && parameter.ParameterType.IsByRef)
            {
                MethodInfo method = parameter.Member as MethodInfo;
                if (method == null || method.GetDefinition() == method)
                {
                    this.AddMessage(I18nMessage.CR0023, parameter.Member, parameter.Name);
                }
            }
        }
    }
}
