﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks if a method can be converted to a property.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1024")]
    public class CR0025MethodShouldBeConvertedIntoProperty : AbstractCheck
    {
        /// <summary>
        /// Initializes a new instance of the
        /// CR0025MethodShouldBeConvertedIntoProperty class.
        /// </summary>
        public CR0025MethodShouldBeConvertedIntoProperty()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the check to react on method parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseMethodStarted += this.OnParseMethodStarted;
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            if (!method.Name.StartsWith("Get", StringComparison.Ordinal))
            {
                return;
            }

            if (method.GetParameters().Length > 0)
            {
                return;
            }

            if (method.ReturnType == typeof(void) || method.ReturnType.IsArray)
            {
                return;
            }

            this.AddMessage(I18nMessage.CR0025, method);
        }
    }
}
