﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that enumerations are mapped to <see cref="int"/>.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1028")]
    public class CR0029EnumShouldBeMappedToInt32 : AbstractCheck
    {
        /// <summary>
        /// Initializes a new instance of the
        /// CR0029EnumShouldBeMappedToInt32 class.
        /// </summary>
        public CR0029EnumShouldBeMappedToInt32()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on type parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            if (type.IsEnum && type.GetEnumUnderlyingType() != typeof(int))
            {
                this.AddMessage(I18nMessage.CR0029, type, type.GetEnumUnderlyingType().FullName);
            }
        }
    }
}
