﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that a property always provides a getter.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1044")]
    public class CR0041PropertyShouldHaveAGetter : AbstractCheck
    {
        /// <summary>
        /// Initializes a new instance of the
        /// CR0041PropertyShouldHaveAGetter class.
        /// </summary>
        public CR0041PropertyShouldHaveAGetter()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on property parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParsedProperty += this.OnParsedProperty;
        }

        /// <summary>
        /// Called when a property is parsed.
        /// </summary>
        /// <param name="property">
        /// The parsed property.
        /// </param>
        private void OnParsedProperty(PropertyInfo property)
        {
            if (!property.CanRead)
            {
                this.AddMessage(I18nMessage.CR0041, property);
            }
        }
    }
}
