﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that all the types are defined into a namespace.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1050")]
    public class CR0046TypeShouldBeDefinedInANamespace : AbstractCheck
    {
        /// <summary>
        /// Initializes a new instance of the
        /// CR0046TypeShouldBeDefinedInANamespace class.
        /// </summary>
        public CR0046TypeShouldBeDefinedInANamespace()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on typey parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            if (string.IsNullOrEmpty(type.Namespace))
            {
                this.AddMessage(I18nMessage.CR0046, type);
            }
        }
    }
}
