﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that fields are private.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1051")]
    public class CR0047FieldShouldBePrivate : AbstractCheck
    {
        /// <summary>
        /// Initializes a new instance of the
        /// CR0047FieldShouldBePrivate class.
        /// </summary>
        public CR0047FieldShouldBePrivate()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on field parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParsedField += this.OnParsedField;
        }

        /// <summary>
        /// Called when a field is parsed.
        /// </summary>
        /// <param name="field">
        /// The parsed field.
        /// </param>
        private void OnParsedField(FieldInfo field)
        {
            // Avoid constant field
            if (field.IsLiteral)
            {
                return;
            }

            // Avoid value field on enumeration
            if (field.DeclaringType.IsEnum)
            {
                return;
            }

            if (!field.IsPrivate)
            {
                this.AddMessage(I18nMessage.CR0047, field);
            }
        }
    }
}
