﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Reflection;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks methods that seems to return Uri - based on their names - avoid
    /// to return string.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1055")]
    public class CR0051MethodReturnUriShouldNotBeString : AbstractCheck
    {
        /// <summary>
        /// Initializes a new instance of the
        /// CR0051MethodReturnUriShouldNotBeString class.
        /// </summary>
        public CR0051MethodReturnUriShouldNotBeString()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on method parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseMethodStarted += this.OnParseMethodStarted;
        }

        /// <summary>
        /// Called when a method is parsed.
        /// </summary>
        /// <param name="method">
        /// The parsed method.
        /// </param>
        private void OnParseMethodStarted(MethodInfo method)
        {
            if (CR0050ParameterUriShouldNotBeString.ContainsUriWord(method.Name)
                && typeof(string).IsAssignableFrom(method.ReturnType))
            {
                this.AddMessage(I18nMessage.CR0051, method);
            }
        }
    }
}
