﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections;
using System.Linq;
using System.Xml;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks the types that inherit from  list of predefined types.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1058")]
    public class CR0053TypeShouldNotInheritFromSomeTypes : AbstractCheck
    {
        /// <summary>
        /// The list of the banned inheritance.
        /// </summary>
        private static readonly Type[] reference = new Type[]
        {
            typeof(ApplicationException),
            typeof(XmlDocument),
            typeof(CollectionBase),
            typeof(DictionaryBase),
            typeof(Queue),
            typeof(ReadOnlyCollectionBase),
            typeof(SortedList),
            typeof(Stack),
        };

        /// <summary>
        /// Initializes a new instance of the
        /// CR0053TypeShouldNotInheritFromSomeTypes class.
        /// </summary>
        public CR0053TypeShouldNotInheritFromSomeTypes()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the check to react on type parsing.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            if (type.BaseType != null && reference.Contains(type.BaseType))
            {
                this.AddMessage(I18nMessage.CR0053, type, type.BaseType.Name);
            }
        }
    }
}
