﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Checks that all the declared exception types are public.
    /// </summary>
    [CheckEquivalence(Constants.FxCop, "CA1064")]
    public class CR0055TypeExceptionShouldBePublic : AbstractCheck
    {
        /// <summary>
        /// The reference to the Exception type.
        /// </summary>
        private static readonly Type reference = typeof(Exception);

        /// <summary>
        /// Initializes a new instance of the
        /// CR0055TypeExceptionShouldBePublic class.
        /// </summary>
        public CR0055TypeExceptionShouldBePublic()
            : base(Constants.Design)
        {
        }

        /// <summary>
        /// Initializes the parser to react on type parsing.
        /// </summary>
        /// <param name="parser">
        /// The current parser.
        /// </param>
        protected override void Initialize(IParser parser)
        {
            parser.ParseTypeStarted += this.OnParseTypeStarted;
        }

        /// <summary>
        /// Called when a type is parsed.
        /// </summary>
        /// <param name="type">
        /// The parsed type.
        /// </param>
        private void OnParseTypeStarted(Type type)
        {
            if (!type.IsPublic && reference.IsAssignableFrom(type))
            {
                this.AddMessage(I18nMessage.CR0055, type);
            }
        }
    }
}
