﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Reflection;

namespace CodeReview.Binary
{
    /// <summary>
    /// Extensions for the <see cref="PropertyInfo"/> class.
    /// </summary>
    public static class PropertyInfoExtensions
    {
        /// <summary>
        /// The format used to create the name of the auto generated field
        /// associated to a property.
        /// </summary>
        private const string BackingFieldFormat = "<{0}>k__BackingField";

        /// <summary>
        /// Analyzes a property and returns a value indicating whether the getter/setter
        /// of the property have been auto-generated at compile-time.
        /// </summary>
        /// <param name="property">
        /// The reference property.
        /// </param>
        /// <returns>
        /// <c>true</c> if the property has auto-generated methods,
        /// <c>false</c> otherwise.
        /// </returns>
        public static bool IsAutoGenerated(this PropertyInfo property)
        {
            string fieldName = string.Format(BackingFieldFormat, property.Name);
            return property.DeclaringType.GetField(fieldName, Parser.AllDeclaredBindingFlags) != null;
        }
    }
}
