﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

namespace CodeReview.Binary
{
    /// <summary>
    /// Defines the available internationalized message.
    /// </summary>
    public enum I18nMessage
    {
        /// <summary>
        /// Associated to CR0001.
        /// </summary>
        CR0001,

        /// <summary>
        /// Associated to CR0002.
        /// </summary>
        CR0002,

        /// <summary>
        /// Associated to CR0003 - when working on a field or a property.
        /// </summary>
        CR0003Member,

        /// <summary>
        /// Associated to CR0003 - when working on a parameter.
        /// </summary>
        CR0003Parameter,

        /// <summary>
        /// Associated to CR0003 - when working on the return of a method.
        /// </summary>
        CR0003Return,

        /// <summary>
        /// Associated to CR0004 - when working on a field.
        /// </summary>
        CR0004Field,

        /// <summary>
        /// Associated to CR0004 - when working on a property.
        /// </summary>
        CR0004Property,

        /// <summary>
        /// Associated to CR0004 - when working on a method.
        /// </summary>
        CR0004Method,

        /// <summary>
        /// Associated with CR0005.
        /// </summary>
        CR0005,

        /// <summary>
        /// Associated with CR0006.
        /// </summary>
        CR0006,

        /// <summary>
        /// Associated to CR0007 - when working on a field.
        /// </summary>
        CR0007Field,

        /// <summary>
        /// Associated to CR0007 - when working on a property.
        /// </summary>
        CR0007Property,

        /// <summary>
        /// Associated to CR0007 - when working on the return type of a method.
        /// </summary>
        CR0007Return,

        /// <summary>
        /// Associated to CR0007 - when working on a parameter.
        /// </summary>
        CR0007Parameter,

        /// <summary>
        /// Associated with CR0008.
        /// </summary>
        CR0008,

        /// <summary>
        /// Associated with CR0009.
        /// </summary>
        CR0009,

        /// <summary>
        /// Associated to CR0010 - when there is no zero field value.
        /// </summary>
        CR0010Missing,

        /// <summary>
        /// Associated to CR0010 - when there is two field with the zero value.
        /// </summary>
        CR0010Double,

        /// <summary>
        /// Associated to CR0011 - when there is no None field.
        /// </summary>
        CR0011Missing,

        /// <summary>
        /// Associated to CR0011 - when the field None has not the zero value.
        /// </summary>
        CR0011Value,

        /// <summary>
        /// Associated with CR0012.
        /// </summary>
        CR0012,

        /// <summary>
        /// Associated with CR0013.
        /// </summary>
        CR0013,

        /// <summary>
        /// Associated with CR0014.
        /// </summary>
        CR0014,

        /// <summary>
        /// Associated with CR0015.
        /// </summary>
        CR0015,

        /// <summary>
        /// Associated with CR0016.
        /// </summary>
        CR0016,

        /// <summary>
        /// Associated with CR0017.
        /// </summary>
        CR0017,

        /// <summary>
        /// Associated with CR0018.
        /// </summary>
        CR0018,

        /// <summary>
        /// Associated to CR0019 - when a positioned argument doesn't have a property.
        /// </summary>
        CR0019Argument,

        /// <summary>
        /// Associated to CR0019 - when a property is not read-only.
        /// </summary>
        CR0019Property,

        /// <summary>
        /// Associated with CR0020.
        /// </summary>
        CR0020,

        /// <summary>
        /// Associated with CR0021.
        /// </summary>
        CR0021,

        /// <summary>
        /// Associated with CR0022.
        /// </summary>
        CR0022,

        /// <summary>
        /// Associated with CR0023.
        /// </summary>
        CR0023,

        /// <summary>
        /// Associated with CR0024.
        /// </summary>
        CR0024,

        /// <summary>
        /// Associated with CR0025.
        /// </summary>
        CR0025,

        /// <summary>
        /// Associated with CR0026.
        /// </summary>
        CR0026,

        /// <summary>
        /// Associated with CR0027.
        /// </summary>
        CR0027,

        /// <summary>
        /// Associated with CR0028.
        /// </summary>
        CR0028,

        /// <summary>
        /// Associated with CR0029.
        /// </summary>
        CR0029,

        /// <summary>
        /// Associated with CR0030.
        /// </summary>
        CR0030,

        /// <summary>
        /// Associated with CR0031.
        /// </summary>
        CR0031,

        /// <summary>
        /// Associated with CR0032.
        /// </summary>
        CR0032,

        /// <summary>
        /// Associated with CR0033.
        /// </summary>
        CR0033,

        /// <summary>
        /// Associated with CR0034.
        /// </summary>
        CR0034,

        /// <summary>
        /// Associated with CR0035.
        /// </summary>
        CR0035,

        /// <summary>
        /// Associated with CR0036.
        /// </summary>
        CR0036,

        /// <summary>
        /// Associated with CR0037.
        /// </summary>
        CR0037,

        /// <summary>
        /// Associated with CR0038.
        /// </summary>
        CR0038,

        /// <summary>
        /// Associated to CR0039 - when working on a type.
        /// </summary>
        CR0039Type,

        /// <summary>
        /// Associated to CR0039 - when working on a constructor.
        /// </summary>
        CR0039Constructor,

        /// <summary>
        /// Associated to CR0039 - when working on a method.
        /// </summary>
        CR0039Method,

        /// <summary>
        /// Associated to CR0039 - when working on a field.
        /// </summary>
        CR0039Field,

        /// <summary>
        /// Associated to CR0039 - when working on a event.
        /// </summary>
        CR0039Event,

        /// <summary>
        /// Associated to CR0039 - when working on a property.
        /// </summary>
        CR0039Property,

        /// <summary>
        /// Associated with CR0040.
        /// </summary>
        CR0040,

        /// <summary>
        /// Associated with CR0041.
        /// </summary>
        CR0041,

        /// <summary>
        /// Associated with CR0042.
        /// </summary>
        CR0042,

        /// <summary>
        /// Associated to CR0043 - when working on a field.
        /// </summary>
        CR0043Field,

        /// <summary>
        /// Associated to CR0043 - when working on a constructor.
        /// </summary>
        CR0043Constructor,

        /// <summary>
        /// Associated to CR0043 - when working on a field.
        /// </summary>
        CR0043Property,

        /// <summary>
        /// Associated to CR0043 - when working on an event.
        /// </summary>
        CR0043Event,

        /// <summary>
        /// Associated to CR0043 - when working on a method.
        /// </summary>
        CR0043Method,

        /// <summary>
        /// Associated to CR0044 - when working on a constructor.
        /// </summary>
        CR0044Constructor,

        /// <summary>
        /// Associated to CR0044 - when working on a field.
        /// </summary>
        CR0044Property,

        /// <summary>
        /// Associated to CR0044 - when working on an event.
        /// </summary>
        CR0044Event,

        /// <summary>
        /// Associated to CR0044 - when working on a method.
        /// </summary>
        CR0044Method,

        /// <summary>
        /// Associated with CR0045.
        /// </summary>
        CR0045,

        /// <summary>
        /// Associated with CR0046.
        /// </summary>
        CR0046,

        /// <summary>
        /// Associated with CR0047.
        /// </summary>
        CR0047,

        /// <summary>
        /// Associated with CR0048.
        /// </summary>
        CR0048,

        /// <summary>
        /// Associated with CR0049.
        /// </summary>
        CR0049,

        /// <summary>
        /// Associated with CR0050.
        /// </summary>
        CR0050,

        /// <summary>
        /// Associated with CR0051.
        /// </summary>
        CR0051,

        /// <summary>
        /// Associated with CR0052.
        /// </summary>
        CR0052,

        /// <summary>
        /// Associated with CR0053.
        /// </summary>
        CR0053,

        /// <summary>
        /// Associated with CR0054.
        /// </summary>
        CR0054,

        /// <summary>
        /// Associated with CR0055.
        /// </summary>
        CR0055,

        /// <summary>
        /// Associated with CR0056.
        /// </summary>
        CR0056,
    }
}
