﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Diagnostics.CodeAnalysis;

namespace CodeReview.Binary
{
    /// <summary>
    /// The delegate used by <see cref="ICheck"/> to register a new issue.
    /// </summary>
    /// <param name="check">
    /// The current check.
    /// </param>
    /// <param name="localeMessage">
    /// The localized error message.
    /// </param>
    /// <param name="target">
    /// The associated target.
    /// </param>
    /// <param name="messageId">
    /// The message identifier - could be <c>null</c>.
    /// </param>
    [SuppressMessage(
            Constants.Design,
            "CR0026:MethodShouldAvoidRepetitiveParameters",
            Justification = "The parameters are not of the same nature")]
    public delegate void AddMessageDelegate(ICheck check, string localeMessage, string target, string messageId);

    /// <summary>
    /// Represents a check that should be valid on a parsed assembly.
    /// </summary>
    /// <remarks>
    /// <para>
    /// When implementing a check, the <see cref="ICheck.Initialize"/>
    /// should be used to link the check to the associated parsing events.
    /// </para>
    /// </remarks>
    public interface ICheck : ICheckDetail
    {
        /// <summary>
        /// Initializes the check for a specific parser.
        /// </summary>
        /// <param name="parser">
        /// The parser associated to the check.
        /// </param>
        /// <param name="addMessage">
        /// The handler used to create a new message if the check is not respected.
        /// </param>
        void Initialize(IParser parser, AddMessageDelegate addMessage);
    }
}
