﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

namespace CodeReview.Binary
{
    /// <summary>
    /// Defines the metadata associated to a check.
    /// </summary>
    public interface ICheckDetail
    {
        /// <summary>
        /// Gets the category of the check.
        /// </summary>
        /// <remarks>
        /// By convention, the values of this property is based on the
        /// namespace of the checks-set followed by the code of the category
        /// like <c>CodeReview.Naming</c>.
        /// </remarks>
        string Category { get; }

        /// <summary>
        /// Gets the code associated with the check.
        /// </summary>
        /// <remarks>
        /// <para>
        /// By convention, the code is composed of two letters followed by
        /// four digits.
        /// </para>
        /// <para>
        /// The letters are associated to the library.
        /// <list type="description">
        /// <item>
        /// <term><c>CR</c></term>
        /// <description>Code Review (this project)</description>
        /// </item>
        /// <item>
        /// <term><c>CA</c></term>
        /// <description>Code Analysis (FxCop)</description>
        /// </item>
        /// <item>
        /// <term><c>SA</c></term>
        /// <description>Source Analysis (SourceCop)</description>
        /// </item>
        /// </list>
        /// </para>
        /// </remarks>
        string CheckId { get; }
    }
}
