﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

namespace CodeReview.Binary
{
    /// <summary>
    /// Defines the metadata associated to a code review message.
    /// </summary>
    /// <remarks>
    /// <para>
    /// A message is considered unique based on its <see cref="ICheckDetail.Category"/>,
    /// <see cref="ICheckDetail.CheckId"/>, <see cref="IMessage.Target"/> and
    /// <see cref="IMessage.MessageId"/> properties - see the special
    /// <see cref="MessageComparer"/> class.
    /// </para>
    /// <para>
    /// The two other properties (<see cref="IMessage.Level"/> and <see cref="IMessage.LocaleMessage"/>)
    /// are considered as secondary data.
    /// </para>
    /// </remarks>
    /// <seealso cref="MessageComparer" />
    public interface IMessage : ICheckDetail
    {
        /// <summary>
        /// Gets the full path of the target.
        /// </summary>
        string Target { get; }

        /// <summary>
        /// Gets the identifier of the message.
        /// </summary>
        /// <remarks>
        /// This property is used to give more details about the issue.
        /// </remarks>
        string MessageId { get; }

        /// <summary>
        /// Gets the level of the message.
        /// </summary>
        /// <remarks>
        /// This property can be set, to be managed outside of the checker.
        /// </remarks>
        MessageLevel Level { get; }

        /// <summary>
        /// Gets the localized message based on the UI Culture.
        /// </summary>
        string LocaleMessage { get; }
    }
}
