﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Collections.Generic;
using System.Collections.ObjectModel;

namespace CodeReview.Command
{
    /// <summary>
    /// Contains the codes associated with an option.
    /// </summary>
    public class OptionCodes
    {
        /// <summary>
        /// The associated option.
        /// </summary>
        private readonly Option option;

        /// <summary>
        /// Initializes a new instance of the OptionCodes class.
        /// </summary>
        /// <param name="option">
        /// The associated option.
        /// </param>
        /// <param name="codes">
        /// The associated codes.
        /// </param>
        public OptionCodes(Option option, params string[] codes)
        {
            this.option = option;
            this.Codes = new ReadOnlyCollection<string>(new List<string>(codes));
        }

        /// <summary>
        /// Gets the associated option.
        /// </summary>
        public Option Option
        {
            get { return this.option; }
        }

        /// <summary>
        /// Gets the associated codes.
        /// </summary>
        public ICollection<string> Codes { get; private set; }
    }
}
