﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;

namespace CodeReview.Command
{
    /// <summary>
    /// Validates the values of an option.
    /// </summary>
    public class OptionValidator
    {
        /// <summary>
        /// The associated option.
        /// </summary>
        private readonly Option option;

        /// <summary>
        /// The method used to validates the option values.
        /// </summary>
        private readonly Func<OptionValues, bool> validator;

        /// <summary>
        /// Initializes a new instance of the OptionValidator class.
        /// </summary>
        /// <param name="option">
        /// The associated option.
        /// </param>
        /// <param name="validator">
        /// The method used to validates the option values.
        /// </param>
        public OptionValidator(Option option, Func<OptionValues, bool> validator)
        {
            this.option = option;
            this.validator = validator;
        }

        /// <summary>
        /// Gets the associated option.
        /// </summary>
        public Option Option
        {
            get { return this.option; }
        }

        /// <summary>
        /// Gets the method used to validates the option values.
        /// </summary>
        public Func<OptionValues, bool> Validator
        {
            get { return this.validator; }
        }
    }
}
