﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary
{
    /// <summary>
    /// Tests the <see cref="AbstractCheck"/> class.
    /// </summary>
    [TestFixture]
    public static class AbstractCheckTest
    {
        /// <summary>
        /// Tests the <see cref="AbstractCheck"/> constructor
        /// with a null category.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public static void ConstructorNullCategory()
        {
            ICheck instance = new TT0123MockCheck(null);
            Assert.IsNotNull(instance);
        }

        /// <summary>
        /// Tests the <see cref="AbstractCheck"/> constructor
        /// with a wrong short id.
        /// </summary>
        [Test]
        [ExpectedException(typeof(NotSupportedException))]
        public static void ConstructorWrongShortCheckId()
        {
            ICheck instance = new WrongMockCheck();
            Assert.IsNotNull(instance);
        }

        /// <summary>
        /// Tests the <see cref="AbstractCheck.CheckId"/> property.
        /// </summary>
        [Test]
        public static void CheckId()
        {
            ICheck instance = new TT0123MockCheck();
            Assert.AreEqual("TT0123:MockCheck", instance.CheckId);
        }

        /// <summary>
        /// Tests the <see cref="AbstractCheck.ShortCheckId"/> property.
        /// </summary>
        [Test]
        public static void ShortCheckId()
        {
            AbstractCheck instance = new TT0123MockCheck();
            Assert.AreEqual("TT0123", instance.ShortCheckId);
        }

        /// <summary>
        /// Tests the <see cref="AbstractCheck.Category"/> property.
        /// </summary>
        [Test]
        public static void Category()
        {
            ICheck instance = new TT0123MockCheck();
            Assert.AreEqual("category", instance.Category);
        }

        /// <summary>
        /// Defines a basic implementation of the <see cref="AbstractCheck"/> class.
        /// </summary>
        private class TT0123MockCheck : AbstractCheck
        {
            /// <summary>
            /// Initializes a new instance of the TT0123MockCheck class.
            /// </summary>
            public TT0123MockCheck()
                : base("category")
            {
            }

            /// <summary>
            /// Initializes a new instance of the TT0123MockCheck class with a specific category.
            /// </summary>
            /// <param name="category">
            /// The category of the check.
            /// </param>
            public TT0123MockCheck(string category)
                : base(category)
            {
            }

            /// <summary>
            /// This method does nothing.
            /// </summary>
            /// <param name="parser">
            /// This parameter is not used.
            /// </param>
            protected override void Initialize(IParser parser)
            {
            }
        }

        /// <summary>
        /// Defines a basic implementation of the <see cref="AbstractCheck"/> class.
        /// </summary>
        private class WrongMockCheck : AbstractCheck
        {
            /// <summary>
            /// Initializes a new instance of the WrongMockCheck class.
            /// </summary>
            public WrongMockCheck()
                : base("category")
            {
            }

            /// <summary>
            /// This method does nothing.
            /// </summary>
            /// <param name="parser">
            /// This parameter is not used.
            /// </param>
            protected override void Initialize(IParser parser)
            {
            }
        }
    }
}
