﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0001TypeAbstractShouldNotHavePublicConstructors"/> class.
    /// </summary>
    [TestFixture]
    public class CR0001TypeAbstractShouldNotHavePublicConstructorsTest : AbstractCheckTest
    {
        /// <summary>
        /// Does a test based on the current check.
        /// </summary>
        [Test]
        public void Check()
        {
            this.Parser.StartParseType(typeof(MockClass));
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockClass), new Type[] { typeof(int) }));
            Assert.IsNull(this.Message);
            this.Parser.StartParseConstructor(Utilities.GetConstructor(typeof(MockClass), Type.EmptyTypes));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initiatizes a new instance of the
        /// <see cref="CR0001TypeAbstractShouldNotHavePublicConstructors"/> class.
        /// </summary>
        /// <returns>
        /// The tested check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0001TypeAbstractShouldNotHavePublicConstructors();
        }

        /// <summary>
        /// Is used for the <see cref="CR0001TypeAbstractShouldNotHavePublicConstructorsTest"/> test.
        /// </summary>
        private abstract class MockClass
        {
            /// <summary>
            /// Initializes a new instance of the MockClass class.
            /// </summary>
            /// <remarks>
            /// This constructor should fail to the check.
            /// </remarks>
            public MockClass()
            {
            }

            /// <summary>
            /// Initializes a new instance of the MockClass class.
            /// </summary>
            /// <param name="p">
            /// The parameter is not used.
            /// </param>
            /// <remarks>
            /// This constructor should pass the check successfully.
            /// </remarks>
            protected MockClass(int p)
            {
            }
        }
    }
}
