﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0005EventShouldBeSystemEventHandler"/> class.
    /// </summary>
    [TestFixture]
    public class CR0005EventShouldBeSystemEventHandlerTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests with events.
        /// </summary>
        [Test]
        public void Event()
        {
            this.Parser.ParseEvent(Utilities.GetEvent(typeof(MockClass), "Normal"));
            Assert.IsNull(this.Message);
            this.Parser.ParseEvent(Utilities.GetEvent(typeof(MockClass), "Generic"));
            Assert.IsNull(this.Message);
            this.Parser.ParseEvent(Utilities.GetEvent(typeof(MockClass), "Other"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes the current test.
        /// </summary>
        /// <returns>
        /// The initialized check instance.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0005EventShouldBeSystemEventHandler();
        }

        /// <summary>
        /// A class used for the test.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// A normal event that should pass the check.
            /// </summary>
            public event System.EventHandler Normal;

            /// <summary>
            /// A generic event that should pass the check.
            /// </summary>
            public event System.EventHandler<MyEventArgs> Generic;

            /// <summary>
            /// A special event that should failed on the check.
            /// </summary>
            public event Action Other;
        }

        /// <summary>
        /// A class used for the definition of the tested events.
        /// </summary>
        private class MyEventArgs : EventArgs
        {
        }
    }
}
