﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0006TypeGenericShouldNotHaveMoreThanTwoParameters"/> class.
    /// </summary>
    [TestFixture]
    public class CR0006TypeGenericShouldNotHaveMoreThanTwoParametersTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check on types.
        /// </summary>
        [Test]
        public void Type()
        {
            this.Parser.StartParseType(typeof(MockClass));
            Assert.IsNull(this.Message);
            this.Parser.StartParseType(typeof(MockGenericClass1<>));
            Assert.IsNull(this.Message);
            this.Parser.StartParseType(typeof(MockGenericClass2<,>));
            Assert.IsNull(this.Message);
            this.Parser.StartParseType(typeof(MockSubclass));
            Assert.IsNull(this.Message);
            this.Parser.StartParseType(typeof(MockGenericClass3<,,>));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initiatizes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The tested check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0006TypeGenericShouldNotHaveMoreThanTwoParameters();
        }

        /// <summary>
        /// A test class that should pass the check.
        /// </summary>
        private class MockClass
        {
        }

        /// <summary>
        /// A test class that should pass the check.
        /// </summary>
        /// <typeparam name="T">
        /// The parameter is not used.
        /// </typeparam>
        private class MockGenericClass1<T>
        {
        }

        /// <summary>
        /// A test class that should pass the check.
        /// </summary>
        /// <typeparam name="S">
        /// The parameter is not used.
        /// </typeparam>
        /// <typeparam name="T">
        /// The parameter is not used.
        /// </typeparam>
        private class MockGenericClass2<S, T>
        {
        }

        /// <summary>
        /// A test class that should failed on the check.
        /// </summary>
        /// <typeparam name="R">
        /// The parameter is not used.
        /// </typeparam>
        /// <typeparam name="S">
        /// The parameter is not used.
        /// </typeparam>
        /// <typeparam name="T">
        /// The parameter is not used.
        /// </typeparam>
        private class MockGenericClass3<R, S, T>
        {
        }

        /// <summary>
        /// A test class that should pass the check.
        /// </summary>
        private class MockSubclass : MockGenericClass3<int, int, int>
        {
        }
    }
}
