﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0008ParameterWithRefShouldUseGenerics"/> class.
    /// </summary>
    [TestFixture]
    public class CR0008ParameterWithRefShouldUseGenericsTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check based on a generic method.
        /// </summary>
        [Test]
        public void CheckGeneric()
        {
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockClass), "Generic"));
            Assert.IsNull(this.Message);
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "Generic", "normal"));
            Assert.IsNull(this.Message);
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "Generic", "generic"));
            Assert.IsNull(this.Message);
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "Generic", "withGeneric"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();
            this.Parser.EndParseMethod(Utilities.GetMethod(typeof(MockClass), "Generic"));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check based on a normal method.
        /// </summary>
        [Test]
        public void CheckNonGeneric()
        {
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockClass), "NonGeneric"));
            Assert.IsNull(this.Message);
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "NonGeneric", "normal"));
            Assert.IsNull(this.Message);
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "NonGeneric", "nonGeneric"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "NonGeneric", "withGeneric"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage(); 
            this.Parser.EndParseMethod(Utilities.GetMethod(typeof(MockClass), "NonGeneric"));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0008ParameterWithRefShouldUseGenerics();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// A generic method.
            /// </summary>
            /// <typeparam name="T">
            /// The parameter is not used.
            /// </typeparam>
            /// <param name="normal">
            /// The parameter is not used.
            /// </param>
            /// <param name="generic">
            /// The parameter is not used.
            /// </param>
            /// <param name="withGeneric">
            /// The parameter is not used.
            /// </param>
            public void Generic<T>(string normal, ref T generic, ref ICollection<T> withGeneric)
            {
            }

            /// <summary>
            /// A non-generic method.
            /// </summary>
            /// <param name="normal">
            /// The parameter is not used.
            /// </param>
            /// <param name="nonGeneric">
            /// The parameter is not used.
            /// </param>
            /// <param name="withGeneric">
            /// The parameter is not used.
            /// </param>
            public void NonGeneric(string normal, ref string nonGeneric, ref ICollection<string> withGeneric)
            {
            }
        }
    }
}
