﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Reflection;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0009MethodGenericShouldUseInference"/> class.
    /// </summary>
    [TestFixture]
    public class CR0009MethodGenericShouldUseInferenceTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests on methods.
        /// </summary>
        [Test]
        public void Method()
        {
            this.ParseMethod(Utilities.GetMethod(typeof(MockClass<>), "NormalMethod"));
            Assert.IsNull(this.Message);
            this.ParseMethod(Utilities.GetMethod(typeof(MockClass<>), "GenericMethodOk"));
            Assert.IsNull(this.Message);
            this.ParseMethod(Utilities.GetMethod(typeof(MockClass<>), "GenericMethodBad"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0009MethodGenericShouldUseInference();
        }

        /// <summary>
        /// Parses a method and all its parameters based on
        /// the <see cref="MockParser"/> methods and events.
        /// </summary>
        /// <param name="method">
        /// The method to be parsed.
        /// </param>
        private void ParseMethod(MethodInfo method)
        {
            this.Parser.StartParseMethod(method);
            foreach (ParameterInfo parameter in method.GetParameters())
            {
                this.Parser.ParseParameter(parameter);
            }

            this.Parser.EndParseMethod(method);
        }

        /// <summary>
        /// A test class.
        /// </summary>
        /// <typeparam name="T">
        /// The parameter is not used.
        /// </typeparam>
        private class MockClass<T>
        {
            /// <summary>
            /// Initializes a new instance of the MockClass class.
            /// </summary>
            public MockClass()
            {
            }

            /// <summary>
            /// A method that should pass the test.
            /// </summary>
            /// <param name="p">
            /// The parameter.
            /// </param>
            /// <returns>
            /// The return type.
            /// </returns>
            public int NormalMethod(int p)
            {
                return p;
            }

            /// <summary>
            /// A method that should pass the test.
            /// </summary>
            /// <param name="p">
            /// The parameter.
            /// </param>
            /// <returns>
            /// The return type.
            /// </returns>
            /// <typeparam name="P">
            /// The generic parameter.
            /// </typeparam>
            public P GenericMethodOk<P>(P p)
            {
                return p;
            }

            /// <summary>
            /// A method that should failed to the test.
            /// </summary>
            /// <returns>
            /// The return type.
            /// </returns>
            /// <typeparam name="P">
            /// The generic parameter.
            /// </typeparam>
            public P GenericMethodBad<P>()
            {
                return default(P);
            }
        }
    }
}
