﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0010EnumShouldHaveZeroValue"/> class.
    /// </summary>
    [TestFixture]
    public class CR0010EnumShouldHaveZeroValueTest : AbstractCheckTest
    {
        /// <summary>
        /// A test enumeration.
        /// </summary>
        private enum MockEnum
        {
            /// <summary>
            /// A test value.
            /// </summary>
            Value0 = 0,

            /// <summary>
            /// A test value.
            /// </summary>
            Value1 = 1,

            /// <summary>
            /// A test value.
            /// </summary>
            Value2 = 2,

            /// <summary>
            /// A test value.
            /// </summary>
            Double = 0,
        }

        /// <summary>
        /// Tests the check on a good enumeration.
        /// </summary>
        [Test]
        public void EnumGood()
        {
            this.Parser.StartParseType(typeof(MockEnum));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockEnum), "Value0"));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockEnum), "Value1"));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockEnum), "Value2"));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockEnum));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check on an enumeration without zero value.
        /// </summary>
        [Test]
        public void EnumMissing()
        {
            this.Parser.StartParseType(typeof(MockEnum));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockEnum), "Value1"));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockEnum), "Value2"));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockEnum));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check on an enumeration containing two zero field.
        /// </summary>
        [Test]
        public void EnumDouble()
        {
            this.Parser.StartParseType(typeof(MockEnum));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockEnum), "Value0"));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockEnum), "Double"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0010EnumShouldHaveZeroValue();
        }
    }
}
