﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0011EnumFlagShouldHaveANoneValue"/> class.
    /// </summary>
    [TestFixture]
    public class CR0011EnumFlagShouldHaveANoneValueTest : AbstractCheckTest
    {
        /// <summary>
        /// A test enumeration - should pass the check.
        /// </summary>
        private enum MockEnum
        {
            /// <summary>
            /// A test value.
            /// </summary>
            Value0 = 0,

            /// <summary>
            /// A test value.
            /// </summary>
            None = 0,

            /// <summary>
            /// A test value.
            /// </summary>
            Value1 = 1,
        }

        /// <summary>
        /// A test enumeration - should pass the check.
        /// </summary>
        [Flags]
        private enum MockFlagsOk
        {
            /// <summary>
            /// A test value.
            /// </summary>
            None = 0,

            /// <summary>
            /// A test value.
            /// </summary>
            Value0 = 0,

            /// <summary>
            /// A test value.
            /// </summary>
            Value1 = 1,
        }

        /// <summary>
        /// A test enumeration - should failed to the check.
        /// </summary>
        [Flags]
        private enum MockFlagsBad1
        {
            /// <summary>
            /// A test value.
            /// </summary>
            Value1 = 1,

            /// <summary>
            /// A test value.
            /// </summary>
            None = 2,
        }

        /// <summary>
        /// A test enumeration - should failed to the check.
        /// </summary>
        [Flags]
        private enum MockFlagsBad2
        {
            /// <summary>
            /// A test value.
            /// </summary>
            Value0 = 0,

            /// <summary>
            /// A test value.
            /// </summary>
            Value1 = 1,
        }

        /// <summary>
        /// Tests the check on enum.
        /// </summary>
        [Test]
        public void Enum()
        {
            this.Parser.StartParseType(typeof(MockEnum));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockEnum), "None"));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockEnum), "Value0"));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockEnum), "Value1"));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check on flag.
        /// </summary>
        [Test]
        public void Flag()
        {
            this.Parser.StartParseType(typeof(MockFlagsOk));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockFlagsOk), "Value0"));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockFlagsOk), "Value1"));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockFlagsOk), "None"));
            Assert.IsNull(this.Message);

            this.Parser.StartParseType(typeof(MockFlagsBad1));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockFlagsBad1), "Value1"));
            Assert.IsNull(this.Message);
            this.Parser.ParseField(Utilities.GetField(typeof(MockFlagsBad1), "None"));
            Assert.IsNotNull(this.Message);

            this.ClearMessage();
            this.Parser.StartParseType(typeof(MockFlagsBad2));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0011EnumFlagShouldHaveANoneValue();
        }
    }
}
