﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0012EventHandlerShouldHaveAProperSignature"/> class.
    /// </summary>
    [TestFixture]
    public class CR0012EventHandlerShouldHaveAProperSignatureTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check with classic events.
        /// </summary>
        [Test]
        public void Event()
        {
            this.Parser.StartParseType(typeof(MockClass.GoodEventHandler));
            Assert.IsNull(this.Message);
            this.Parser.StartParseType(typeof(MockClass.BadEventHandler));
            Assert.IsNull(this.Message);
            this.Parser.ParseEvent(Utilities.GetEvent(typeof(MockClass), "StandardEvent"));
            Assert.IsNull(this.Message);
            this.Parser.ParseEvent(Utilities.GetEvent(typeof(MockClass), "GoodEvent"));
            Assert.IsNull(this.Message);
            this.Parser.ParseEvent(Utilities.GetEvent(typeof(MockClass), "BadEvent"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check with generic events.
        /// </summary>
        [Test]
        public void GenericEvent()
        {
            this.Parser.StartParseType(typeof(MockClass.GoodGenericEventHandler<>));
            Assert.IsNull(this.Message);
            this.Parser.StartParseType(typeof(MockClass.BadGenericEventHandler<>));
            Assert.IsNull(this.Message);
            this.Parser.ParseEvent(Utilities.GetEvent(typeof(MockClass), "StandardGenericEvent"));
            Assert.IsNull(this.Message);
            this.Parser.ParseEvent(Utilities.GetEvent(typeof(MockClass), "GoodGenericEvent"));
            Assert.IsNull(this.Message);
            this.Parser.ParseEvent(Utilities.GetEvent(typeof(MockClass), "BadGenericEvent"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0012EventHandlerShouldHaveAProperSignature();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// A test delegate.
            /// </summary>
            /// <param name="sender">
            /// The sender of the event.
            /// </param>
            /// <param name="e">
            /// The arguments or the event.
            /// </param>
            public delegate void GoodEventHandler(object sender, MyEventArgs e);

            /// <summary>
            /// A test delegate.
            /// </summary>
            /// <param name="sender">
            /// The sender of the event.
            /// </param>
            /// <param name="e">
            /// The arguments or the event.
            /// </param>
            /// <typeparam name="T">
            /// The type of the event argument.
            /// </typeparam>
            public delegate void GoodGenericEventHandler<T>(object sender, T e) where T : MyEventArgs;

            /// <summary>
            /// A test delegate.
            /// </summary>
            /// <param name="e">
            /// The arguments or the event.
            /// </param>
            public delegate void BadEventHandler(MyEventArgs e);

            /// <summary>
            /// A test delegate.
            /// </summary>
            /// <param name="e">
            /// The arguments or the event.
            /// </param>
            /// <typeparam name="T">
            /// The type of the event argument.
            /// </typeparam>
            public delegate void BadGenericEventHandler<T>(T e) where T : MyEventArgs;

            /// <summary>
            /// A test event.
            /// </summary>
            public event EventHandler StandardEvent;

            /// <summary>
            /// A test event.
            /// </summary>
            public event EventHandler<MyEventArgs> StandardGenericEvent;

            /// <summary>
            /// A test event.
            /// </summary>
            public event GoodEventHandler GoodEvent;

            /// <summary>
            /// A test event.
            /// </summary>
            public event GoodGenericEventHandler<MyEventArgs> GoodGenericEvent;

            /// <summary>
            /// A test event.
            /// </summary>
            public event BadEventHandler BadEvent;

            /// <summary>
            /// A test event.
            /// </summary>
            public event BadGenericEventHandler<MyEventArgs> BadGenericEvent;

            /// <summary>
            /// A test event args class.
            /// </summary>
            public class MyEventArgs : EventArgs
            {
            }
        }
    }
}
