﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections;
using System.Collections.Generic;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0013TypeEnumerableShouldBeGeneric"/> class.
    /// </summary>
    [TestFixture]
    public class CR0013TypeEnumerableShouldBeGenericTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check with types.
        /// </summary>
        [Test]
        public void Type()
        {
            this.Parser.StartParseType(typeof(MockGoodCollection));
            Assert.IsNull(this.Message);
            this.Parser.StartParseType(typeof(MockGoodGenericCollection<>));
            Assert.IsNull(this.Message);
            this.Parser.StartParseType(typeof(MockDictionary));
            Assert.IsNull(this.Message);
            this.Parser.StartParseType(typeof(MockWrongCollection));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0013TypeEnumerableShouldBeGeneric();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockWrongCollection : IEnumerable
        {
            /// <summary>
            /// Does nothing.
            /// </summary>
            /// <returns>
            /// Always raise a <see cref="NotImplementedException"/> exception.
            /// </returns>
            public IEnumerator GetEnumerator()
            {
                throw new NotImplementedException();
            }
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockGoodCollection : List<string>
        {
        }

        /// <summary>
        /// A test class.
        /// </summary>
        /// <typeparam name="T">
        /// The parameter is not used.
        /// </typeparam>
        private class MockGoodGenericCollection<T> : List<T>
            where T : MockWrongCollection
        {
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockDictionary : Dictionary<string, string>
        {
        }
    }
}
