﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Reflection;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0014TypeWithAddOrSubstractShouldImplementEquality"/> class.
    /// </summary>
    [TestFixture]
    public class CR0014TypeWithAddOrSubstractShouldImplementEqualityTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check with a good type.
        /// </summary>
        [Test]
        public void CheckGood()
        {
            this.Parser.StartParseType(typeof(GoodMockClass));
            Assert.IsNull(this.Message);
            foreach (MethodInfo method in typeof(GoodMockClass).GetMethods(CodeReview.Binary.Parser.AllDeclaredBindingFlags))
            {
                this.Parser.StartParseMethod(method);
                Assert.IsNull(this.Message);
            }

            this.Parser.EndParseType(typeof(GoodMockClass));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check with a bad type.
        /// </summary>
        [Test]
        public void CheckBad()
        {
            this.Parser.StartParseType(typeof(BadMockClass));
            Assert.IsNull(this.Message);
            foreach (MethodInfo method in typeof(BadMockClass).GetMethods(CodeReview.Binary.Parser.AllDeclaredBindingFlags))
            {
                this.Parser.StartParseMethod(method);
                Assert.IsNull(this.Message);
            }

            this.Parser.EndParseType(typeof(BadMockClass));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes the tested check.
        /// </summary>
        /// <returns>
        /// The tested check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0014TypeWithAddOrSubstractShouldImplementEquality();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class GoodMockClass
        {
            /// <summary>
            /// A test operator.
            /// </summary>
            /// <param name="a">
            /// The parameter is not used.
            /// </param>
            /// <param name="b">
            /// The parameter is not used.
            /// </param>
            /// <returns>
            /// Always return null.
            /// </returns>
            public static GoodMockClass operator +(GoodMockClass a, GoodMockClass b)
            {
                return null;
            }

            /// <summary>
            /// A test operator.
            /// </summary>
            /// <param name="a">
            /// The parameter is not used.
            /// </param>
            /// <param name="b">
            /// The parameter is not used.
            /// </param>
            /// <returns>
            /// Always return false.
            /// </returns>
            public static bool operator ==(GoodMockClass a, GoodMockClass b)
            {
                return false;
            }

            /// <summary>
            /// A test operator.
            /// </summary>
            /// <param name="a">
            /// The parameter is not used.
            /// </param>
            /// <param name="b">
            /// The parameter is not used.
            /// </param>
            /// <returns>
            /// Always return false.
            /// </returns>
            public static bool operator !=(GoodMockClass a, GoodMockClass b)
            {
                return false;
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <returns>
            /// Return the base definition of the method.
            /// </returns>
            public override int GetHashCode()
            {
                return base.GetHashCode();
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="obj">
            /// The parameter is not used.
            /// </param>
            /// <returns>
            /// Return the base definition of the method.
            /// </returns>
            public override bool Equals(object obj)
            {
                return base.Equals(obj);
            }
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class BadMockClass
        {
            /// <summary>
            /// A test operator.
            /// </summary>
            /// <param name="a">
            /// The parameter is not used.
            /// </param>
            /// <param name="b">
            /// The parameter is not used.
            /// </param>
            /// <returns>
            /// Always return null.
            /// </returns>
            public static BadMockClass operator -(BadMockClass a, BadMockClass b)
            {
                return null;
            }
        }
    }
}
