﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Reflection;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0024TypeShouldNotExposeMultidimensionalIndexer"/> class.
    /// </summary>
    [TestFixture]
    public class CR0024TypeShouldNotExposeMultidimensionalIndexerTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check with bad indexers.
        /// </summary>
        [Test]
        public void CheckBad()
        {
            PropertyInfo[] properties = typeof(MockBadClass).GetProperties();
            Assert.AreEqual(2, properties.Length);
            foreach (PropertyInfo property in properties)
            {
                this.Parser.ParseProperty(property);
                Assert.IsNotNull(this.Message);
                this.ClearMessage();
            }
        }

        /// <summary>
        /// Tests the check with a good indexer.
        /// </summary>
        [Test]
        public void CheckGood()
        {
            PropertyInfo[] properties = typeof(MockGoodClass).GetProperties();
            Assert.AreEqual(1, properties.Length);
            foreach (PropertyInfo property in properties)
            {
                this.Parser.ParseProperty(property);
                Assert.IsNull(this.Message);
            }
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0024TypeShouldNotExposeMultidimensionalIndexer();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockBadClass
        {
            /// <summary>
            /// An indexer.
            /// </summary>
            /// <param name="x">The first parameter.</param>
            /// <param name="y">The second parameter.</param>
            /// <returns>The return value.</returns>
            public double this[double x, double y]
            {
                get
                {
                    return x * y;
                }
            }

            /// <summary>
            /// An indexer.
            /// </summary>
            /// <param name="x">The first parameter.</param>
            /// <param name="y">The second parameter.</param>
            /// <param name="z">The third parameter.</param>
            /// <returns>The return value.</returns>
            public double this[double x, double y, double z]
            {
                get
                {
                    return x * y * z;
                }
            }
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockGoodClass
        {
            /// <summary>
            /// An indexer.
            /// </summary>
            /// <param name="x">The first parameter.</param>
            /// <returns>The return value.</returns>
            public double this[double x]
            {
                get
                {
                    return x;
                }
            }
        }
    }
}
