﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0025MethodShouldBeConvertedIntoProperty"/> class.
    /// </summary>
    [TestFixture]
    public class CR0025MethodShouldBeConvertedIntoPropertyTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check.
        /// </summary>
        [Test]
        public void Check()
        {
            this.Parser.ParseMethod(Utilities.GetMethod(typeof(MockClass), "GetArray"));
            Assert.IsNull(this.Message);
            this.Parser.ParseMethod(Utilities.GetMethod(typeof(MockClass), "GetInfoWithParameter"));
            Assert.IsNull(this.Message);
            this.Parser.ParseMethod(Utilities.GetMethod(typeof(MockClass), "GetInfo"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0025MethodShouldBeConvertedIntoProperty();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// A test method.
            /// </summary>
            /// <returns>
            /// The return type.
            /// </returns>
            public string GetInfo()
            {
                return null;
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="para">
            /// A parameter.
            /// </param>
            /// <returns>
            /// The return type.
            /// </returns>
            public string GetInfoWithParameter(string para)
            {
                return para;
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <returns>
            /// The return type.
            /// </returns>
            public string[] GetArray()
            {
                return null;
            }
        }
    }
}
