﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0029EnumShouldBeMappedToInt32"/> class.
    /// </summary>
    [TestFixture]
    public class CR0029EnumShouldBeMappedToInt32Test : AbstractCheckTest
    {
        /// <summary>
        /// A test enumeration that passes the check.
        /// </summary>
        private enum GoodEnum
        {
            /// <summary>
            /// A test value.
            /// </summary>
            First,
        }

        /// <summary>
        /// A test enumeration that fails on the check.
        /// </summary>
        private enum BadEnum : uint
        {
            /// <summary>
            /// A test value.
            /// </summary>
            First,
        }

        /// <summary>
        /// Tests the check.
        /// </summary>
        [Test]
        public void Check()
        {
            this.Parser.StartParseType(typeof(GoodEnum));
            Assert.IsNull(this.Message);
            this.Parser.StartParseType(typeof(BadEnum));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0029EnumShouldBeMappedToInt32();
        }
    }
}
