﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0030MethodShouldBeReplacedByEvent"/> class.
    /// </summary>
    [TestFixture]
    public class CR0030MethodShouldBeReplacedByEventTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check on a valid method starting with Raise.
        /// </summary>
        [Test]
        public void CheckValid()
        {
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockClass), "RaiseEvent"));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check on methods with invalid names.
        /// </summary>
        /// <param name="methodName">
        /// The name of the tested method.
        /// </param>
        [TestCase("AddOnMethod")]
        [TestCase("RemoveOnMethod")]
        [TestCase("FireMethod")]
        [TestCase("RaiseMethod")]
        public void CheckInvalid(string methodName)
        {
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockClass), methodName));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0030MethodShouldBeReplacedByEvent();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// A test event.
            /// </summary>
            public event System.EventHandler Event;

            /// <summary>
            /// A test method that should be redesigned to use an event.
            /// </summary>
            public void AddOnMethod()
            {
            }

            /// <summary>
            /// A test method that should be redesigned to use an event.
            /// </summary>
            public void RemoveOnMethod()
            {
            }

            /// <summary>
            /// A test method that should be redesigned to use an event.
            /// </summary>
            public void FireMethod()
            {
            }

            /// <summary>
            /// A test method that should be redesigned to use an event.
            /// </summary>
            public void RaiseMethod()
            {
            }

            /// <summary>
            /// A test method that is correct because associated to an existing event.
            /// </summary>
            public void RaiseEvent()
            {
                if (this.Event != null)
                {
                    this.Event(this, EventArgs.Empty);
                }
            }
        }
    }
}
