﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0034TypeICollectionShouldBeStronglyTyped"/> class.
    /// </summary>
    [TestFixture]
    public class CR0034TypeICollectionShouldBeStronglyTypedTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check on a valid collection.
        /// </summary>
        [Test]
        public void CheckValid()
        {
            this.Parser.StartParseType(typeof(MockCollection));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockCollection), "CopyTo", new Type[] { typeof(string[]), typeof(int) }));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockCollection), "CopyTo", new Type[] { typeof(Array), typeof(int) }));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockCollection));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check on an invalid collection.
        /// </summary>
        [Test]
        public void CheckInvalid()
        {
            this.Parser.StartParseType(typeof(MockCollection));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockCollection), "CopyTo", new Type[] { typeof(Array), typeof(int) }));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockCollection));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0034TypeICollectionShouldBeStronglyTyped();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockCollection : ICollection
        {
            /// <summary>
            /// Gets a test value.
            /// </summary>
            public int Count
            {
                get { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets a value indicating whether the collection is synchronized.
            /// </summary>
            public bool IsSynchronized
            {
                get { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets a test value.
            /// </summary>
            public object SyncRoot
            {
                get { throw new NotImplementedException(); }
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="array">The parameter is not used.</param>
            /// <param name="index">The parameter is not used.</param>
            public void CopyTo(string[] array, int index)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="array">The parameter is not used.</param>
            /// <param name="index">The parameter is not used.</param>
            public void CopyTo(Array array, int index)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <returns>Always raise an exception.</returns>
            public IEnumerator GetEnumerator()
            {
                throw new NotImplementedException();
            }
        }
    }
}
