﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0035TypeIEnumeratorShouldBeStronglyTyped"/> class.
    /// </summary>
    [TestFixture]
    public class CR0035TypeIEnumeratorShouldBeStronglyTypedTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check with a valid enumerator.
        /// </summary>
        [Test]
        public void CheckValid()
        {
            this.Parser.StartParseType(typeof(MockEnumerator));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockEnumerator), "Current"));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockEnumerator));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check with an invalid enumerator.
        /// </summary>
        [Test]
        public void CheckInvalid()
        {
            this.Parser.StartParseType(typeof(MockEnumerator));
            Assert.IsNull(this.Message);
            this.Parser.EndParseType(typeof(MockEnumerator));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0035TypeIEnumeratorShouldBeStronglyTyped();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockEnumerator : IEnumerator
        {
            /// <summary>
            /// Gets a test value.
            /// </summary>
            object IEnumerator.Current
            {
                get { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets a test value.
            /// </summary>
            public string Current
            {
                get { throw new NotImplementedException(); }
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <returns>Always raise an exception.</returns>
            public bool MoveNext()
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test method.
            /// </summary>
            public void Reset()
            {
                throw new NotImplementedException();
            }
        }
    }
}
