﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0040IndexerShouldUseStandardType"/> class.
    /// </summary>
    [TestFixture]
    public class CR0040IndexerShouldUseStandardTypeTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check with simple indexers.
        /// </summary>
        [Test]
        public void CheckSimple()
        {
            this.Parser.ParseProperty(Utilities.GetIndexer(typeof(MockClass), typeof(int)));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetIndexer(typeof(MockClass), typeof(long)));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetIndexer(typeof(MockClass), typeof(string)));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetIndexer(typeof(MockClass), typeof(object)));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Tests the check with multi-indexers.
        /// </summary>
        [Test]
        public void CheckMulti()
        {
            this.Parser.ParseProperty(Utilities.GetIndexer(typeof(MockClass), typeof(int), typeof(int)));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetIndexer(typeof(MockClass), typeof(int), typeof(object)));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0040IndexerShouldUseStandardType();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// Gets or sets a test value.
            /// </summary>
            /// <param name="index">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public string this[int index]
            {
                get { throw new NotImplementedException(); }
                set { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets or sets a test value.
            /// </summary>
            /// <param name="index">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public string this[long index]
            {
                get { throw new NotImplementedException(); }
                set { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets or sets a test value.
            /// </summary>
            /// <param name="index">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public string this[string index]
            {
                get { throw new NotImplementedException(); }
                set { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets or sets a test value.
            /// </summary>
            /// <param name="index">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public string this[object index]
            {
                get { throw new NotImplementedException(); }
                set { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets or sets a test value.
            /// </summary>
            /// <param name="index1">The parameter is not used.</param>
            /// <param name="index2">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public string this[int index1, int index2]
            {
                get { throw new NotImplementedException(); }
                set { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets or sets a test value.
            /// </summary>
            /// <param name="index1">The parameter is not used.</param>
            /// <param name="index2">The parameter is not used.</param>
            /// <returns>Always raise an exception.</returns>
            public string this[int index1, object index2]
            {
                get { throw new NotImplementedException(); }
                set { throw new NotImplementedException(); }
            }
        }
    }
}
