﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0041PropertyShouldHaveAGetter"/> class.
    /// </summary>
    [TestFixture]
    public class CR0041PropertyShouldHaveAGetterTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check.
        /// </summary>
        [Test]
        public void Check()
        {
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockClass), "Getter"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockClass), "GetterSetter"));
            Assert.IsNull(this.Message);
            this.Parser.ParseProperty(Utilities.GetProperty(typeof(MockClass), "Setter"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0041PropertyShouldHaveAGetter();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// Gets a test value.
            /// </summary>
            public string Getter
            {
                get { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets or sets a test value.
            /// </summary>
            public string GetterSetter { get; set; }

            /// <summary>
            /// Sets a test value.
            /// </summary>
            public string Setter
            {
                set { throw new NotImplementedException(); }
            }
        }
    }
}
