﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0047FieldShouldBePrivate"/> class.
    /// </summary>
    [TestFixture]
    public class CR0047FieldShouldBePrivateTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check.
        /// </summary>
        [Test]
        public void Check()
        {
            this.Parser.ParseField(Utilities.GetField(typeof(MockClass), "privateField"));
            Assert.IsNull(this.Message);

            this.Parser.ParseField(Utilities.GetField(typeof(MockClass), "Constant"));
            Assert.IsNull(this.Message);

            this.Parser.ParseField(Utilities.GetField(typeof(MockClass), "InternalField"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();

            this.Parser.ParseField(Utilities.GetField(typeof(MockClass), "PublicField"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0047FieldShouldBePrivate();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// A test constant.
            /// </summary>
            public const string Constant = "test";

            /// <summary>
            /// A test field.
            /// </summary>
            public string PublicField;

            /// <summary>
            /// A test field.
            /// </summary>
            internal string InternalField;

            /// <summary>
            /// A test field.
            /// </summary>
            private string privateField;

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public string Property
            {
                get { return this.privateField; }
                set { this.privateField = value; }
            }
        }
    }
}
