﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Linq;
using System.Reflection;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0050ParameterUriShouldNotBeString"/> class.
    /// </summary>
    [TestFixture]
    public class CR0050ParameterUriShouldNotBeStringTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check.
        /// </summary>
        [Test]
        public void CheckInterceptedNames()
        {
            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "Method1", "uri"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();

            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "Method2", "url"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();

            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "Method3", "urn"));
            Assert.IsNotNull(this.Message);
            this.ClearMessage();

            this.Parser.ParseParameter(Utilities.GetParameter(typeof(MockClass), "Method4", "test"));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check.
        /// </summary>
        [Test]
        public void CheckInterceptedTypes()
        {
            Type type = typeof(MockClass);
            ParameterInfo instance;

            instance = type.GetMethod("Method", new Type[] { typeof(double) }).GetParameters().FirstOrDefault();
            this.Parser.ParseParameter(instance);
            Assert.IsNull(this.Message);

            instance = type.GetMethod("Method", new Type[] { typeof(Uri) }).GetParameters().FirstOrDefault();
            this.Parser.ParseParameter(instance);
            Assert.IsNull(this.Message);

            instance = type.GetMethod("Method", new Type[] { typeof(string) }).GetParameters().FirstOrDefault();
            this.Parser.ParseParameter(instance);
            Assert.IsNull(this.Message);

            instance = type.GetMethod("Method1", new Type[] { typeof(string) }).GetParameters().FirstOrDefault();
            this.Parser.ParseParameter(instance);
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0050ParameterUriShouldNotBeString();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="stringUri">The parameter is not used.</param>
            public void Method(string stringUri)
            {
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="uriUri">The parameter is not used.</param>
            public void Method(Uri uriUri)
            {
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="doubleUri">The parameter is not used.</param>
            public void Method(double doubleUri)
            {
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="uri">The parameter is not used.</param>
            public void Method1(string uri)
            {
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="url">The parameter is not used.</param>
            public void Method2(string url)
            {
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="urn">The parameter is not used.</param>
            public void Method3(string urn)
            {
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="test">The parameter is not used.</param>
            public void Method4(string test)
            {
            }
        }
    }
}
