﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0051MethodReturnUriShouldNotBeString"/> class.
    /// </summary>
    [TestFixture]
    public class CR0051MethodReturnUriShouldNotBeStringTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check.
        /// </summary>
        [Test]
        public void Check()
        {
            this.Parser.ParseMethod(Utilities.GetMethod(typeof(MockClass), "GoodMethodUri1"));
            Assert.IsNull(this.Message);
            this.Parser.ParseMethod(Utilities.GetMethod(typeof(MockClass), "GoodMethodUri2"));
            Assert.IsNull(this.Message);
            this.Parser.ParseMethod(Utilities.GetMethod(typeof(MockClass), "BadMethodUri"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0051MethodReturnUriShouldNotBeString();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// A test method.
            /// </summary>
            public void GoodMethodUri1()
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <returns>The method always throws an exception.</returns>
            public Uri GoodMethodUri2()
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test method.
            /// </summary>
            /// <returns>The method always throws an exception.</returns>
            public string BadMethodUri()
            {
                throw new NotImplementedException();
            }
        }
    }
}
