﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections;
using System.Reflection;
using System.Xml;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0053TypeShouldNotInheritFromSomeTypes"/> class.
    /// </summary>
    [TestFixture]
    public class CR0053TypeShouldNotInheritFromSomeTypesTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check on classes that inherit from the wrong types.
        /// </summary>
        [Test]
        public void CheckBadClasses()
        {
            bool test = false;
            foreach (Type type in this.GetType().GetNestedTypes(BindingFlags.NonPublic | BindingFlags.Static))
            {
                if (type.Name.StartsWith("BadMockClass"))
                {
                    test = true;
                    this.Parser.StartParseType(type);
                    Assert.IsNotNull(this.Message);
                    this.ClearMessage();
                }
            }

            Assert.IsNull(this.Message);
            Assert.IsTrue(test);
        }

        /// <summary>
        /// Tests the check on good classes.
        /// </summary>
        [Test]
        public void CheckGoodClasses()
        {
            this.Parser.StartParseType(typeof(GoodMockClass));
            Assert.IsNull(this.Message);
            this.Parser.StartParseType(typeof(GoodMockClass1));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0053TypeShouldNotInheritFromSomeTypes();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class BadMockClass1 : ApplicationException
        {
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class BadMockClass2 : XmlDocument
        {
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class BadMockClass3 : CollectionBase
        {
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class BadMockClass4 : DictionaryBase
        {
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class BadMockClass5 : Queue
        {
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class BadMockClass6 : ReadOnlyCollectionBase
        {
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class BadMockClass7 : SortedList
        {
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class BadMockClass8 : Stack
        {
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class GoodMockClass
        {
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class GoodMockClass1 : BadMockClass1
        {
        }
    }
}
