﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0054MethodShouldNotHideBaseMethod"/> class.
    /// </summary>
    [TestFixture]
    public class CR0054MethodShouldNotHideBaseMethodTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check.
        /// </summary>
        [Test]
        public void Check()
        {
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockBaseClass), "Method"));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockClass), "Method", new Type[] { typeof(string) }));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockClass), "Method", new Type[] { typeof(string), typeof(string) }));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockClass), "Method", new Type[] { typeof(string), typeof(object) }));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockClass), "Method", new Type[] { typeof(object), typeof(object) }));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0054MethodShouldNotHideBaseMethod();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockBaseClass
        {
            /// <summary>
            /// A test method - pass the check.
            /// </summary>
            /// <param name="input1">The parameter is not used.</param>
            /// <param name="input2">The parameter is not used.</param>
            public void Method(string input1, object input2)
            {
                throw new NotImplementedException();
            }
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass : MockBaseClass
        {
            /// <summary>
            /// A test method - pass the check.
            /// </summary>
            /// <param name="input">The parameter is not used.</param>
            public void Method(string input)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test method - pass the check.
            /// </summary>
            /// <param name="input1">The parameter is not used.</param>
            /// <param name="input2">The parameter is not used.</param>
            public void Method(string input1, string input2)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test method - pass the check.
            /// </summary>
            /// <param name="input1">The parameter is not used.</param>
            /// <param name="input2">The parameter is not used.</param>
            public new void Method(string input1, object input2)
            {
                throw new NotImplementedException();
            }

            /// <summary>
            /// A test method - failed the check.
            /// </summary>
            /// <param name="input1">The parameter is not used.</param>
            /// <param name="input2">The parameter is not used.</param>
            public void Method(object input1, object input2)
            {
                throw new NotImplementedException();
            }
        }
    }
}
