﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Runtime.InteropServices;
using NUnit.Framework;

namespace CodeReview.Binary.Design
{
    /// <summary>
    /// Tests the <see cref="CR0056MethodNativeShouldBeGrouped"/> class.
    /// </summary>
    [TestFixture]
    public class CR0056MethodNativeShouldBeGroupedTest : AbstractCheckTest
    {
        /// <summary>
        /// Tests the check on a valid class.
        /// </summary>
        [Test]
        public void CheckValid()
        {
            this.Parser.StartParseType(typeof(NativeMethods));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(NativeMethods), "RemoveDirectory"));
            Assert.IsNull(this.Message);
        }

        /// <summary>
        /// Tests the check on a invalid class.
        /// </summary>
        [Test]
        public void CheckInvalid()
        {
            this.Parser.StartParseType(typeof(MockClass));
            Assert.IsNull(this.Message);
            this.Parser.StartParseMethod(Utilities.GetMethod(typeof(MockClass), "RemoveDirectory"));
            Assert.IsNotNull(this.Message);
        }

        /// <summary>
        /// Initializes a new instance of the check.
        /// </summary>
        /// <returns>
        /// The initialized check.
        /// </returns>
        protected override ICheck InitializeCheck()
        {
            return new CR0056MethodNativeShouldBeGrouped();
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private static class MockClass
        {
            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="name">The directory to remove.</param>
            /// <returns><c>true</c> if the directory has been removed.</returns>
            [DllImport("kernel32.dll", CharSet = CharSet.Unicode)]
            internal static extern bool RemoveDirectory(string name);
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private static class NativeMethods
        {
            /// <summary>
            /// A test method.
            /// </summary>
            /// <param name="name">The directory to remove.</param>
            /// <returns><c>true</c> if the directory has been removed.</returns>
            [DllImport("kernel32.dll", CharSet = CharSet.Unicode)]
            internal static extern bool RemoveDirectory(string name);
        }
    }
}
