﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary
{
    /// <summary>
    /// Tests the <see cref="Engine"/> class.
    /// </summary>
    [TestFixture]
    public static class EngineTest
    {
        /// <summary>
        /// Tests the <see cref="Engine.Messages"/> property.
        /// </summary>
        [Test]
        public static void Messages()
        {
            Engine instance = new Engine();
            Assert.IsNotNull(instance.Messages);
            Assert.AreEqual(0, instance.Messages.Length);
        }

        /// <summary>
        /// Tests the <see cref="Engine.Register"/> method.
        /// </summary>
        [Test]
        public static void Register()
        {
            MockCheck check = new MockCheck();
            Engine instance = new Engine();
            instance.Register(check);
            instance.DoAnalysis(MockAssembly.Instance);

            Assert.IsTrue(check.IsInitialized);
        }

        /// <summary>
        /// Tests the <see cref="Engine.DoAnalysis"/> method.
        /// </summary>
        [Test]
        public static void DoAnalysis()
        {
            MockCheck check = new MockCheck();
            Engine instance = new Engine();
            instance.Register(check);
            instance.DoAnalysis(MockAssembly.Instance);

            Assert.IsTrue(check.IsParsed);
        }

        /// <summary>
        /// Defines a basic implementation of the <see cref="ICheck"/> class.
        /// </summary>
        private class MockCheck : ICheck
        {
            /// <summary>
            /// Gets the category of the check.
            /// </summary>
            public string Category
            {
                get { return "category"; }
            }

            /// <summary>
            /// Gets the full code of the check.
            /// </summary>
            public string CheckId
            {
                get { return "checkid"; }
            }

            /// <summary>
            /// Gets a value indicating whether the Initialize method has been invoked.
            /// </summary>
            public bool IsInitialized { get; private set; }

            /// <summary>
            /// Gets a value indicating whether a parse has been done.
            /// </summary>
            public bool IsParsed { get; private set; }

            /// <summary>
            /// Adds a message and associates the parser to a simple action.
            /// </summary>
            /// <param name="parser">
            /// The parser used with the check.
            /// </param>
            /// <param name="addMessage">
            /// The parameter is not used.
            /// </param>
            public void Initialize(IParser parser, AddMessageDelegate addMessage)
            {
                this.IsInitialized = true;
                parser.ParseTypeEnded += new TypeHandler(this.OnEndParseType);
            }

            /// <summary>
            /// Called at the end of a type parsing.
            /// </summary>
            /// <param name="type">
            /// The parameter is not used.
            /// </param>
            private void OnEndParseType(Type type)
            {
                this.IsParsed = true;
            }
        }
    }
}
