﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;

namespace CodeReview.Binary.Extensions
{
    /// <summary>
    /// Tests specifically the <see cref="MethodInfoExtensions.HasVirtualKeyword"/> method.
    /// </summary>
    [TestFixture]
    public static class MethodInfoExtensionsHasVirtualKeywordTest
    {
        /// <summary>
        /// A test interface.
        /// </summary>
        private interface IMockInterface
        {
            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            string Property { get; set; }

            /// <summary>
            /// A test method.
            /// </summary>
            void Method();
        }

        /// <summary>
        /// Tests the <see cref="System.Reflection.MethodBase.IsVirtual"/> property
        /// and demonstrates that this property is not related to the virtal keyword.
        /// </summary>
        [Test]
        public static void IsVirtual()
        {
            Assert.IsFalse(typeof(MockClass1).GetProperty("Property").GetGetMethod().IsVirtual);
            Assert.IsTrue(typeof(MockClass2).GetProperty("Property").GetGetMethod().IsVirtual);
            Assert.IsTrue(typeof(MockClass3).GetProperty("Property").GetGetMethod().IsVirtual);
            Assert.IsTrue(typeof(MockClass4).GetProperty("Property").GetGetMethod().IsVirtual);
            Assert.IsTrue(typeof(MockClass5).GetProperty("Property").GetGetMethod().IsVirtual);

            Assert.IsFalse(typeof(MockClass1).GetMethod("Method").IsVirtual);
            Assert.IsTrue(typeof(MockClass2).GetMethod("Method").IsVirtual);
            Assert.IsTrue(typeof(MockClass3).GetMethod("Method").IsVirtual);
            Assert.IsTrue(typeof(MockClass4).GetMethod("Method").IsVirtual);
            Assert.IsTrue(typeof(MockClass5).GetMethod("Method").IsVirtual);
        }

        /// <summary>
        /// Tests the <see cref="MethodInfoExtensions.HasVirtualKeyword"/> method.
        /// </summary>
        [Test]
        public static void HasVirtualKeyword()
        {
            Assert.IsFalse(typeof(MockClass1).GetProperty("Property").GetGetMethod().HasVirtualKeyword());
            Assert.IsTrue(typeof(MockClass2).GetProperty("Property").GetGetMethod().HasVirtualKeyword());
            Assert.IsFalse(typeof(MockClass3).GetProperty("Property").GetGetMethod().HasVirtualKeyword());
            Assert.IsTrue(typeof(MockClass4).GetProperty("Property").GetGetMethod().HasVirtualKeyword());
            Assert.IsFalse(typeof(MockClass5).GetProperty("Property").GetGetMethod().HasVirtualKeyword());

            Assert.IsFalse(typeof(MockClass1).GetMethod("Method").HasVirtualKeyword());
            Assert.IsTrue(typeof(MockClass2).GetMethod("Method").HasVirtualKeyword());
            Assert.IsFalse(typeof(MockClass3).GetMethod("Method").HasVirtualKeyword());
            Assert.IsTrue(typeof(MockClass4).GetMethod("Method").HasVirtualKeyword());
            Assert.IsFalse(typeof(MockClass5).GetMethod("Method").HasVirtualKeyword());
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass1
        {
            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public string Property { get; set; }

            /// <summary>
            /// A test method.
            /// </summary>
            public void Method()
            {
                throw new NotImplementedException();
            }
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass2
        {
            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public virtual string Property { get; set; }

            /// <summary>
            /// A test method.
            /// </summary>
            public virtual void Method()
            {
                throw new NotImplementedException();
            }
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass3 : IMockInterface
        {
            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public string Property { get; set; }

            /// <summary>
            /// A test method.
            /// </summary>
            public void Method()
            {
                throw new NotImplementedException();
            }
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass4 : IMockInterface
        {
            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public virtual string Property { get; set; }

            /// <summary>
            /// A test method.
            /// </summary>
            public virtual void Method()
            {
                throw new NotImplementedException();
            }
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass5 : MockClass4
        {
            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public override string Property { get; set; }

            /// <summary>
            /// A test method.
            /// </summary>
            public override void Method()
            {
                throw new NotImplementedException();
            }
        }
    }
}
