﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;

namespace CodeReview.Binary.Extensions
{
    /// <summary>
    /// Tests the <see cref="PropertyInfoExtensions"/> class.
    /// </summary>
    [TestFixture]
    public static class PropertyInfoExtensionsTest
    {
        /// <summary>
        /// Tests the <see cref="PropertyInfoExtensions.IsAutoGenerated"/> method.
        /// </summary>
        [Test]
        public static void IsAutoGenerated()
        {
            Assert.IsTrue(Utilities.GetProperty(typeof(MockClass), "PublicAutoProperty").IsAutoGenerated());
            Assert.IsTrue(Utilities.GetProperty(typeof(MockClass), "ProtectedAutoProperty").IsAutoGenerated());
            Assert.IsFalse(Utilities.GetProperty(typeof(MockClass), "PublicProperty").IsAutoGenerated());
            Assert.IsFalse(Utilities.GetProperty(typeof(MockClass), "ProtectedProperty").IsAutoGenerated());
        }

        /// <summary>
        /// A test class.
        /// </summary>
        private class MockClass
        {
            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public string PublicProperty
            {
                get { throw new NotImplementedException(); }
                set { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            public string PublicAutoProperty { get; set; }

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            protected string ProtectedProperty
            {
                get { throw new NotImplementedException(); }
                set { throw new NotImplementedException(); }
            }

            /// <summary>
            /// Gets or sets a test property.
            /// </summary>
            protected string ProtectedAutoProperty { get; set; }
        }
    }
}
