﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using NUnit.Framework;

namespace CodeReview.Binary
{
    /// <summary>
    /// Tests the <see cref="I18nManager"/> class.
    /// </summary>
    [TestFixture]
    public static class I18nManagerTest
    {
        /// <summary>
        /// Tests the <see cref="I18nManager.GetString(Type,string)"/> method.
        /// </summary>
        [Test]
        public static void GetStringTypeKey()
        {
            string actual;

            // Test with a not existing text
            actual = I18nManager.GetString(typeof(MockI18nMessage), "None");
            Assert.IsNull(actual);

            // Test with an existing text
            actual = I18nManager.GetString(typeof(MockI18nMessage), "MyMessage");
            Assert.IsNotNullOrEmpty(actual);
            Assert.AreEqual("my message", actual);
        }

        /// <summary>
        /// Tests the <see cref="I18nManager.GetString(Enum)"/> method.
        /// </summary>
        [Test]
        public static void GetStringEnum()
        {
            string actual;

            // Test with a not existing text
            actual = I18nManager.GetString(MockI18nMessage.None);
            Assert.IsNull(actual);

            // Test with an existing text
            actual = I18nManager.GetString(MockI18nMessage.MyMessage);
            Assert.IsNotNullOrEmpty(actual);
            Assert.AreEqual("my message", actual);
        }

        /// <summary>
        /// Tests the <see cref="I18nManager.Format(Enum,object[])"/> method.
        /// </summary>
        [Test]
        public static void Format()
        {
            string actual;

            // Test with a not existing text
            actual = I18nManager.Format(MockI18nMessage.None);
            Assert.IsNull(actual);

            // Test with an existing text
            actual = I18nManager.Format(MockI18nMessage.MyFormat, "p1");
            Assert.IsNotNullOrEmpty(actual);
            Assert.AreEqual("my format <p1>", actual);
        }
    }
}
