﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Linq;
using System.Reflection;

namespace CodeReview.Binary
{
    /// <summary>
    /// Encapsulates the loading of the mock assembly used for tests.
    /// </summary>
    internal static class MockAssembly
    {
        /// <summary>
        /// The unique instance of the assembly.
        /// </summary>
        private static Assembly instance;

        /// <summary>
        ///  Gets the unique instance of the assembly.
        /// </summary>
        public static Assembly Instance
        {
            get
            {
                if (instance == null)
                {
                    instance = Assembly.ReflectionOnlyLoadFrom("CodeReview.MockAssembly.dll");
                }

                return instance;
            }
        }

        /// <summary>
        /// Retrieves the first attribute data for a specific type of attribute.
        /// </summary>
        /// <param name="type">
        ///  The type of attribute to retrieve.
        /// </param>
        /// <returns>
        ///  The data of the attribute.
        /// </returns>
        /// <exception cref="InvalidOperationException">
        /// There is no attribute of the specified type.
        /// </exception>
        public static CustomAttributeData Attribute(Type type)
        {
            return Instance.GetCustomAttributesData().First(a => a.Constructor.DeclaringType == type);
        }
    }
}
