﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System.Text;
using NUnit.Framework;

namespace CodeReview.Binary
{
    /// <summary>
    /// Tests the <see cref="StrongName"/> class.
    /// </summary>
    [TestFixture]
    public static class StrongNameTest
    {
        /// <summary>
        /// Tests the <see cref="StrongName.PublicKey"/> constant.
        /// </summary>
        [Test]
        public static void PublicKey()
        {
            byte[] key = typeof(StrongName).Assembly.GetName().GetPublicKey();
            string expected = Convert(key);
            string actual = StrongName.PublicKey;
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        /// Tests the <see cref="StrongName.PublicKeyToken"/> constant.
        /// </summary>
        [Test]
        public static void PublicKeyToken()
        {
            byte[] token = typeof(StrongName).Assembly.GetName().GetPublicKeyToken();
            string expected = Convert(token);
            string actual = StrongName.PublicKeyToken;
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        /// Converts an array of bytes to a string.
        /// </summary>
        /// <param name="value">
        /// The array of bytes.
        /// </param>
        /// <returns>
        /// The hexadecimal representation of <paramref name="value"/>.
        /// </returns>
        private static string Convert(byte[] value)
        {
            StringBuilder builder = new StringBuilder(value.Length);
            foreach (byte b in value)
            {
                builder.AppendFormat("{0:x2}", b);
            }

            return builder.ToString();
        }
    }
}
