﻿/*
Copyright © 2010 François Karman

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

See <license.txt> for the complete license of the software.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using NUnit.Framework;

namespace CodeReview.Command
{
    /// <summary>
    /// Tests the <see cref="OptionFactory"/> class.
    /// </summary>
    [TestFixture]
    public static class OptionFactoryTest
    {
        /// <summary>
        /// Tests the <see cref="OptionFactory.Codes"/> property.
        /// </summary>
        [Test]
        public static void Codes()
        {
            Option[] options = Enum.GetValues(typeof(Option)).Cast<Option>().ToArray();
            Option[] codes = OptionFactory.Codes.Select(c => c.Option).ToArray();

            // Compare length but omit the Option.None value
            CollectionAssert.DoesNotContain(codes, Option.None);
            CollectionAssert.AllItemsAreUnique(codes);
            CollectionAssert.AllItemsAreUnique(options);
            Assert.AreEqual(options.Length - 1, codes.Length);

            // Compare options to codes
            // Look for missing option in the codes definition
            foreach (Option option in options)
            {
                if (option == Option.None)
                {
                    CollectionAssert.DoesNotContain(codes, option);
                }
                else
                {
                    CollectionAssert.Contains(codes, option);
                }
            }

            // Compare codes to options - should never failed
            foreach (Option code in codes)
            {
                CollectionAssert.Contains(options, code);
            }
        }

        /// <summary>
        /// Tests the <see cref="OptionFactory.RetrieveOption"/> method.
        /// </summary>
        [Test]
        public static void RetrieveOption()
        {
            IDictionary<string, Option> codes = new Dictionary<string, Option>();
            foreach (OptionCodes optionCode in OptionFactory.Codes)
            {
                Assert.IsNotNull(optionCode);
                Assert.IsNotNull(optionCode.Option);

                foreach (string code in optionCode.Codes)
                {
                    CollectionAssert.DoesNotContain(codes.Keys, code);
                    codes.Add(code, optionCode.Option);
                }
            }

            foreach (KeyValuePair<string, Option> pair in codes)
            {
                Assert.AreEqual(pair.Value, OptionFactory.RetrieveOption(pair.Key));
            }
        }
    }
}
